'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _lang = require('./lang');

var _lang2 = _interopRequireDefault(_lang);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var encode = encodeURIComponent;
var decode = function decode(qs) {
  return decodeURIComponent(qs).replace(/\+/g, ' ');
};

/**
 * Convert a collection to a query string
 * @param {Object|Map|Array} query
 * @return {String}
 */
var stringify = function stringify(query) {
  var entries = Array.isArray(query) ? query : _lang2.default.entries(query);
  var serialize = function serialize(entry) {
    return entry.slice(0, 2).map(encode).join('=');
  };

  return entries.map(serialize).join('&');
};

/**
 * Explode a query string on its boundaries and strip falsy values
 * @param {String} qs
 * @param {String} delimiter
 * @return {Array}
 */
var tokenize = function tokenize(qs) {
  var boundary = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '&';
  return qs.split(boundary).filter(Boolean);
};

var parse = {};

/**
 * Convert a query string to a list of entries:
 * @example
 *   parse.entries({foo: 1, bar: 2})
 *   >>> [['foo', '1'], ['bar', '2']]
 * @param {String} qs
 * @return {Array}
 */
parse.entries = function (qs) {
  // strip leading question mark
  if (qs && qs[0] === '?') {
    qs = qs.slice(1);
  }

  return tokenize(qs).reduce(function (out, pair) {
    out.push(pair.split('=').slice(0, 2).map(decode));
    return out;
  }, []);
};

/**
 * Convert a query string to a Map
 * @param {String} qs
 * @return {Map}
 */
parse.map = function (qs) {
  return new Map(parse.entries(qs));
};

/**
 * Convert a query string to a plain object
 * @param {String} qs
 * @return {Object}
 */
parse.object = function (qs) {
  var reducer = function reducer(out, entry) {
    var _entry = _slicedToArray(entry, 2),
        k = _entry[0],
        v = _entry[1];

    out[k] = v;
    return out;
  };

  return _lang2.default.reduce(parse.entries(qs), reducer, {});
};

exports.default = {
  stringify: stringify,
  parse: parse
};
module.exports = exports['default'];