'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Translator = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _tree = require('./tree');

var _components = require('./components');

var _lang = require('./lang');

var _lang2 = _interopRequireDefault(_lang);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @param {String} translation
 * @param {Array} sub
 * @return {String}
 */
var Translator = exports.Translator = function (_Component) {
  _inherits(Translator, _Component);

  _createClass(Translator, [{
    key: 'defaults',


    /**
     * Return configuration defaults
     * @param void
     * @return {Object}
     */
    value: function defaults() {
      return {
        // Tree instance that contains translated messages
        translations: new _tree.Tree(),
        // reducer function that substitutes placeholders
        substitute: function substitute(translation, sub) {
          var _sub = _slicedToArray(sub, 2),
              key = _sub[0],
              value = _sub[1];

          var regex = new RegExp(':' + key, 'g');
          return translation.replace(regex, value);
        }
      };
    }
  }]);

  function Translator() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Translator);

    var _this = _possibleConstructorReturn(this, (Translator.__proto__ || Object.getPrototypeOf(Translator)).call(this, options));

    _this.translations = _this.option('translations', { pull: true });
    return _this;
  }

  /**
   * Tokenize the given expression on '.' and traverse the translation tree
   * until a match is found. If no match is found, return an empty string.
   * This is the main
   * @param {String} expr
   * @param {Object} subs
   * @return {String}
   */


  _createClass(Translator, [{
    key: 'translate',
    value: function translate(expr) {
      var subs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var translations = this.translations;
      var suppress = this.options.suppress;


      try {
        return this.substitute(translations.path(expr), subs);
      } catch (e) {
        return expr;
      }
    }

    /**
     * @param {String} translation
     * @param {Object} subs - string substitutions
     * @return {String}
     */

  }, {
    key: 'substitute',
    value: function substitute(translation) {
      var subs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (!translation) {
        throw new Error('expected translation');
      }

      if (_lang2.default.empty(subs)) {
        return translation;
      }

      var substitute = this.options.substitute;


      return _lang2.default.entries(subs).reduce(substitute, translation);
    }
  }]);

  return Translator;
}(_components.Component);

exports.default = {
  Translator: Translator
};