import sort from '../../src/sort';
const noop = () => {};

describe('sort', () => {

  describe('.asc()', () => {
    it('accepts a value resolver function', () => {
      expect(sort.asc(() => {})).not.to.throw();
    })

    it('returns a sort comparator', () => {
      expect(sort.asc(noop)).to.be.a('function');
    });

    it('comparator sorts in ascending order', () => {
      let values = [{prop: 3}, {prop: 1}, {prop: 2}];
      let evaluate = value => value.prop;
      let comparator = sort.asc(evaluate);
      let sorted = values.sort(comparator);

      expect(sorted[0].prop).to.equal(1);
      expect(sorted[1].prop).to.equal(2);
      expect(sorted[2].prop).to.equal(3);
    });
  });

  describe('.desc()', () => {
    it('accepts a value resolver function', () => {
      expect(sort.desc(() => {})).not.to.throw();
    })

    it('returns a sort comparator', () => {
      expect(sort.desc(noop)).to.be.a('function');
    });

    it('comparator sorts in descending order', () => {
      let values = [{prop: 3}, {prop: 1}, {prop: 2}];
      let evaluate = value => value.prop;
      let comparator = sort.desc(evaluate);
      let sorted = values.sort(comparator);

      expect(sorted[0].prop).to.equal(3);
      expect(sorted[1].prop).to.equal(2);
      expect(sorted[2].prop).to.equal(1);
    });
  });
});
