import {Translator} from '../../src/translation';

describe('translation', () => {
  describe('Translator', () => {

    describe('#translate()', () => {

      it('translates', () => {
        let tree = {path: stub()};
        let translator = new Translator({translations: tree});
        let expr = 'path/to/translation';
        let subs = {a: 'hello', b: 'world'};

        tree.path.withArgs(expr).returns(':a :b');

        let expected = 'hello world';
        let actual = translator.translate(expr, subs);

        expect(tree.path.called).to.be.ok;
        expect(actual).to.equal(expected); 
      });

      it('returns the translation if tree.path throws (without subs)', () => {
        let tree = {path: stub().throws()};
        let translator = new Translator({translations: tree});
        let expr = 'the-translation';

        let expected = expr;
        let actual = translator.translate(expr);

        expect(tree.path.called).to.be.ok;
        expect(actual).to.equal(expected); 
      });

      it('returns the translation if tree.path throws (with subs)', () => {
        let tree = {path: stub().throws()};
        let translator = new Translator({translations: tree});
        let expr = 'the-translation';
        let subs = {a: 'hello', b: 'world'};

        let expected = expr;
        let actual = translator.translate(expr, subs);

        expect(tree.path.called).to.be.ok;
        expect(actual).to.equal(expected); 
      });

    });
  });
});

